#include "cs_surface.h"

int extrusion (struct snurbs* surf, struct cnurbs curve,
				GLfloat* W, GLfloat d)
{
	GLfloat dist[4];
	GLfloat V[4]={0,0,1,1}; // ordine 2 in v (interpolazione lineare)
	GLfloat* CP;
	GLfloat len = sqrt(pow(W[0],2)+pow(W[1],2)+pow(W[2],2));
	int i,j;

	if ((len == 0) || (d<=0))
		return 0;
	if ((curve.dim != 3) && (curve.dim != 4))
		return 0;
	CP = malloc(curve.dim*curve.ncp*2*sizeof(GLfloat));
	for (i=0;i<3;i++) // calcolo del vettore profondit
		dist[i]=W[i]/len*d;
	dist[3]=0;
	for (i=0;i<curve.ncp;i++)  /* control points cp[i][0]=cpcurve[i]*w[i] cp[i][1]=cpcurve[i]*w[i]+W[i] */
		for (j=0;j<curve.dim;j++)
		{
			CP[curve.dim*i+j]=curve.CP[curve.dim*i+j];
			CP[curve.dim*(curve.ncp+i)+j]=curve.CP[curve.dim*i+j]+dist[j];
		}

	surf->CP = CP;
	surf->dim = curve.dim;
	surf->ncpu = curve.ncp;
	surf->ncpv = 2;
	surf->nu = curve.nk;
	surf->nv = 4;
	surf->orderu = curve.order;
	surf->orderv = 2;
	surf->U = malloc(curve.nk*sizeof(GLfloat));
	for (i=0;i<curve.nk;i++)
		surf->U[i] = curve.K[i];
	surf->V = malloc(4*sizeof(GLfloat));
	for (i=0;i<4;i++)
		surf->V[i] = V[i];
	return 1;
}

int ruled(struct snurbs* surf, GLfloat* CPcurve1, GLfloat* CPcurve2, GLfloat* U, int nu, int ncpc,
			   int dim)
{
	GLfloat V[4]={0,0,1,1}; // ordine 2 in v (interpolazione lineare)
	int order = nu - ncpc;
	GLfloat* CP;
	int i,j;

	if ((dim != 3) && (dim != 4))
		return 0;

	CP = malloc(dim*ncpc*2*sizeof(GLfloat));
	for (i=0;i<ncpc;i++) // CP[i][0]=CPcurve1[i]*w1[i] CP[i][1]=CPcurve2[i]*w2[i]
		for (j=0;j<dim;j++)
		{
			CP[dim*i+j]=CPcurve1[dim*i+j];
			CP[dim*(ncpc+i)+j]=CPcurve2[dim*i+j];
		}

	surf->CP = CP;
	surf->dim = dim;
	surf->ncpu = ncpc;
	surf->ncpv = 2;
	surf->nu = nu;
	surf->nv = 4;
	surf->orderu = order;
	surf->orderv = 2;
	surf->U = malloc(nu*sizeof(GLfloat));
	for (i=0;i<nu;i++)
		surf->U[i] = U[i];
	surf->V = malloc(4*sizeof(GLfloat));
	for (i=0;i<4;i++)
		surf->V[i] = V[i];
	return 1;
}

int rotation(struct snurbs* surf, struct cnurbs curve)
{
	// LAB CG3 : implementare superfici NURBS di rotazione
	// INPUT: curve 
	// OUTPUT surf
	return 1;
}

void find_node_V(GLfloat* CP, int n, int ncpc, int orderv, int dim, GLfloat* v_n, GLfloat* V)
// Trova il vettore nodale V e i punti v_n del parametro in cui le curve interpolano i punti di controllo
// Input:
// CP[n][ncpc][dim] punti di controllo delle n curve sezione
// n numero di curve sezione
// ncpc numero di punti di controllo di ogni curva sezione
// orderv ordine delle curve in v
// dim dimensione dello spazio
// Output:
// v_n[n] vettore del parametro v in cui le curve interpolano i punti di controllo
// V[nv] vettore nodale in v
{
	int i,j,k,l;
	GLfloat sum,d;
	GLfloat v;
	int nv = n + orderv;
	
	v_n[0]=0;
	v_n[n-1]=1;
	for (k=1;k<n-1;k++)	// calcolo di v_n
	{
		sum=0;
		for (i=0;i<ncpc;i++)
		{
			d=0;
			for (j=1;j<n;j++)
			{
				v=0;
				for (l=0;l<dim;l++)
					v+=pow(CP[dim*(ncpc*j+i)+l]-CP[dim*(ncpc*(j-1)+i)+l],2);
				d+=sqrt(v);
			}
			v=0;
			for (l=0;l<dim;l++)
				v+=pow(CP[dim*(ncpc*k+i)+l]-CP[dim*(ncpc*(k-1)+i)+l],2);
			sum+=sqrt(v)/d;
		}
		v_n[k] = v_n[k-1] + sum/ncpc;
	}

	for (i=0;i<orderv;i++) // calcolo del vettore nodale V
		V[i]=0;
	for (j=0;j<n-orderv;j++)
	{
		sum=0;
		for (i=j;i<j+orderv-1;i++)
			sum+=v_n[i+1];
		V[j+orderv]=sum/(orderv-1);
	}
	for (i=nv-orderv;i<nv;i++)
		V[i]=1;
}

void sislin(GLfloat* A, GLfloat* b, int n, int dim, GLfloat* x)
// Risolve un sistema lineare Ax=b con algoritmo di Gauss con pivoting parziale
// risolve parallelamente il sistema per una serie di vettori di termini noti b
// Input:
// A[n][n] matrice del sistema lineare
// b[n][dim] vettori dei termini noti
// n dimensione del sistema
// dim numero di vettori dei termini noti
// Output:
// x[n][dim] vettori delle soluzioni
{
	GLfloat* AA=malloc(n*n*sizeof(GLfloat));
	GLfloat* bb=malloc(n*dim*sizeof(GLfloat));
	int i,j,k;
	GLfloat max,temp,m;
	int linea;

	for (i=0;i<n;i++)
		for (j=0;j<n;j++)
			AA[n*i+j]=A[n*i+j];
	for (i=0;i<n;i++)
		for (j=0;j<dim;j++)
			bb[dim*i+j]=b[dim*i+j];
// algoritmo di Gauss con pivoting parziale	
	for (i=0;i<n;i++)
	{
		max = fabs(AA[n*i+i]);
		linea = i;
		for (j=i+1;j<n;j++)
		{
			if (fabs(AA[n*j+i])>max)
			{
				linea = j;
				max = fabs(AA[n*j+i]);
			}
		}
		if (linea>i)
		{
			for (j=i;j<n;j++)
			{
				temp = AA[n*i+j];
				AA[n*i+j] = AA[n*linea+j];
				AA[n*linea+j] = temp;
			}
			for (j=0;j<dim;j++)
			{
				temp = bb[dim*i+j];
				bb[dim*i+j] = bb[dim*linea+j];
				bb[dim*linea+j] = temp;
			}
		}
		for (j=i+1;j<n;j++)
		{
			m = AA[n*j+i]/AA[n*i+i];
			AA[n*j+i] = 0;
			for (k=i+1;k<n;k++)
				AA[n*j+k] -= m*AA[n*i+k];
			for (k=0;k<dim;k++)
				bb[dim*j+k] -= m*bb[dim*i+k];
		}
	}
// sostituzione all'indietro
	for (k=0;k<dim;k++)
		for (i=n-1;i>=0;i--)
		{
			temp = 0;
			for (j=i+1;j<n;j++)
				temp += AA[n*i+j]*x[dim*j+k];
			x[dim*i+k] = (bb[dim*i+k]-temp)/AA[n*i+i];
		}
	free(AA);
	free(bb);
}

void curve_interp(GLfloat* P, GLfloat* u_n, int n, int dim, GLfloat* V, int nv, GLfloat* Q)
// Calcola i punti di controllo di una curva che interpola un insieme dato di punti
// Input:
// P[n][dim] punti da interpolare
// u_n[n] valori del parametro per cui la curva interpola i punti dati
// n numero di punti da interpolare
// dim dimensione dello spazio
// V[nv] vettore nodale della curva interpolante
// nv numero di nodi
// Output:
// Q[n][dim] punti di controllo
{
	int orderv = nv - n;
	int node;
	int i,j;
	GLfloat* base = malloc(orderv*sizeof(GLfloat));
	GLfloat* A = malloc(n*n*sizeof(GLfloat));
// inizializzazione della matrice A
	for (i=0;i<n;i++)
		for(j=0;j<n;j++)
			A[n*i+j]=0;
// creazione della matrice A usando le basi spline
	for (i=0;i<n;i++)
	{
		node = find_spline_base(V,nv,u_n[i],orderv,base);
		for (j=0;j<orderv;j++)
			A[n*i+(j+node-orderv+1)]=base[j];
	}
// risoluzione del sistema lineare per il calcolo dei punti di controllo
	sislin(A,P,n,dim,Q);
	free(base);
	free(A);
}

int skinning(struct snurbs* surf, GLfloat** CPcurve, int n, GLfloat* U, int nu, int ncpc, 
				  GLfloat* dist, int orderv, int dim)
{
	int orderu = nu - ncpc;
	GLfloat* CP,*v_n,*V,*P,*Q,*CQ;
	int i,j,k;
	int nv;

	if (orderv>n)
		return 0;
	if ((dim != 3) && (dim != 4))
		return 0;
	nv = n + orderv;
	CP = malloc(dim*ncpc*n*sizeof(GLfloat));
	CQ = malloc(dim*ncpc*n*sizeof(GLfloat));
	v_n = malloc(n*sizeof(GLfloat));
	V = malloc(nv*sizeof(GLfloat));
	P = malloc(dim*n*sizeof(GLfloat));
	Q = malloc(dim*n*sizeof(GLfloat));
// inizializzazione della matrice dei punti di controllo delle curve sezione
	if (dim == 3)
		for (i=0;i<n;i++)
			for (j=0;j<ncpc;j++)
			{
				CP[dim*(ncpc*i+j)]=CPcurve[i][3*j];
				CP[dim*(ncpc*i+j)+1]=CPcurve[i][3*j+1];
				CP[dim*(ncpc*i+j)+2]=dist[i];
			}
	else
		for (i=0;i<n;i++)
			for (j=0;j<ncpc;j++)
			{
				CP[dim*(ncpc*i+j)]=CPcurve[i][4*j];
				CP[dim*(ncpc*i+j)+1]=CPcurve[i][4*j+1];
				CP[dim*(ncpc*i+j)+2]=dist[i];
				CP[dim*(ncpc*i+j)+3]=CPcurve[i][4*j+3];
			}
// calcolo del vettore nodale V del vettore v_n dei valori del parametro per cui si ha interpolazione
	find_node_V(CP,n,ncpc,orderv,dim,v_n,V);

	for (i=0;i<ncpc;i++)
	{
		for (j=0;j<n;j++)
			for (k=0;k<dim;k++)
				P[dim*j+k]=CP[dim*(ncpc*j+i)+k];
		// calcolo dei punti di controllo della superficie interpolante
		curve_interp(P,v_n,n,dim,V,nv,Q);

		for (j=0;j<n;j++)
			for (k=0;k<dim;k++)
				CQ[dim*(ncpc*j+i)+k]=Q[dim*j+k];
	}

	free(CP);
	free(v_n);
	free(P);
	free(Q);

	surf->CP = CQ;
	surf->dim = dim;
	surf->ncpu = ncpc;
	surf->ncpv = n;
	surf->nu = nu;
	surf->nv = nv;
	surf->orderu = orderu;
	surf->orderv = orderv;
	surf->U = malloc(nu*sizeof(GLfloat));
	for (i=0;i<nu;i++)
		surf->U[i] = U[i];
	surf->V = V;

	return 1;
}

int swinging(struct snurbs* surf, struct cnurbs p, struct cnurbs t, GLfloat s)
{
	int i,j;
	int dim;
	GLfloat* Q;

	if (p.dim != t.dim)
		return 0;
	dim = p.dim;
	if ((dim != 3) && (dim != 4))
		return 0;

	Q = malloc(p.ncp*t.ncp*dim*sizeof(GLfloat));
	if (dim == 4)
		for (i=0;i<p.ncp;i++)
			for (j=0;j<t.ncp;j++)
			{
				Q[dim*(t.ncp*i+j)] = p.CP[dim*i]*t.CP[dim*j]*s; /* Q[i][j][x]=sP[i][x]*T[j][x]*w1[i]*w2[j] */
				Q[dim*(t.ncp*i+j)+1] = p.CP[dim*i]*t.CP[dim*j+1]*s; /* Q[i][j][y]=sP[i][x]*T[j][y]*w1[i]*w2[j] */
				Q[dim*(t.ncp*i+j)+2] = p.CP[dim*i+2]; /* Q[i][j][z]=P[i][z]*w1[i]*w2[j] */
				Q[dim*(t.ncp*i+j)+3] = p.CP[dim*i+3]*t.CP[dim*j+3]; /* Q[i][j][w]=w1[i]*w2[j] */
			}
	else
		for (i=0;i<p.ncp;i++)
			for (j=0;j<t.ncp;j++)
			{
				Q[dim*(t.ncp*i+j)] = p.CP[dim*i]*t.CP[dim*j]*s; /* Q[i][j][x]=sP[i][x]*T[j][x] */
				Q[dim*(t.ncp*i+j)+1] = p.CP[dim*i]*t.CP[dim*j+1]*s; /* Q[i][j][y]=sP[i][x]*T[j][y] */
				Q[dim*(t.ncp*i+j)+2] = p.CP[dim*i+2]; /* Q[i][j][z]=P[i][z] */
			}

	surf->CP = Q;
	surf->dim = dim;
	surf->ncpu = t.ncp;
	surf->ncpv = p.ncp;
	surf->nu = t.nk;
	surf->nv = p.nk;
	surf->orderu = t.order;
	surf->orderv = p.order;
	surf->V = malloc(p.nk*sizeof(GLfloat));
	for (i=0;i<p.nk;i++)
		surf->V[i] = p.K[i];
	surf->U = malloc(t.nk*sizeof(GLfloat));
	for (i=0;i<t.nk;i++)
		surf->U[i] = t.K[i];
	return 1;
}