#pragma once

#ifndef _CS_SURFACE_H
#define _CS_SURFACE_H

#include <math.h>
#include <malloc.h>
#include "glut.h"

#include "snurbs.h"
#include "spline.h"

#ifdef __cplusplus
extern "C" {
#endif

int extrusion (struct snurbs* surf, struct cnurbs curve,
				GLfloat* W, GLfloat d);
// Calcola la superficie estrusa data una curva ed una profondit (razionale e non)
// Restituisce la struttura snurbs che rappresenta la superficie
// Input:
// curve oggetto che rappresenta la curva sezione
// W[3] vettore direzione della profondit
// d modulo della profondit
// Output:
// surf oggetto che rappresenta la superficie
// ritorna 0 se c' stato un errore

int ruled(struct snurbs* surf, GLfloat* CPcurve1, GLfloat* CPcurve2, GLfloat* U, int nu, int ncpc,
			   int dim);
// Calcola la superficie rigata date due curve sezione con lo stesso numero di punti di controllo
// e lo stesso vettore nodale (razionale e non)
// Restituisce la struttura snurbs che rappresenta la superficie
// Input:
// CPcurve1[ncpc][4] punti di controllo della prima curva sezione
// CPcurve2[ncpc][4] punti di controllo della seconda curva sezione
// U[nu] vettore nodale (uguale per le due curve)
// nu numero di nodi (uguale per le 2 curve)
// ncpc numero di punti di controllo (uguale per le 2 curve)
// dim dimensione dei punti (3: non razionale 4: razionale)
// Output:
// surf oggetto che rappresenta la superficie
// ritorna 0 se c' stato un errore

int rotation(struct snurbs* surf, struct cnurbs curve);
// Calcola la superficie di rotazione data una curva profilo (razionale e non)
// Restituisce la struttura snurbs che rappresenta la superficie
// Input:
// curve oggetto che rappresenta la curva profilo (sul piano xz)
// Output:
// surf oggetto che rappresenta la superficie
// ritorna 0 se c' stato un errore

int skinning(struct snurbs* surf, GLfloat** CPcurve, int n, GLfloat* U, int nu, int ncpc, 
				  GLfloat* dist, int orderv, int dim);
// Calcola la superficie skinning data una serie di curve sezione con lo stesso numero di
// punti di controllo e lo stesso vettore nodale in u (razionale e non)
// Restituisce la struttura snurbs che rappresenta la superficie
// Input:
// CPcurve[n][ncpc][2] punti di controllo delle curve (piane)
// n numero delle curve sezione
// U[nu] vettore nodale (uguale per tutte)
// nu numero di nodi (uguale per tutte)
// ncpc numero di punti di controllo (uguale per tutte)
// dist vettore delle coordinate z delle curve
// ordine in direzione v (arbitrario ma orderv<=n)
// dim dimensione dei punti (3: non razionale 4: razionale)
// Output:
// surf oggetto che rappresenta la superficie
// ritorna 0 se c' stato un errore
int swinging(struct snurbs* surf, struct cnurbs p, struct cnurbs t, GLfloat s);
// Calcola la superficie swinging data una curva profilo P e una curva traiettoria T (razionale e non)
// Restituisce la struttura snurbs che rappresenta la superficie
// Input:
// p oggetto che rappresenta la curva profilo (sul piano xz)
// t oggetto che rappresenta la curva traiettoria (sul piano xy, chiusa)
// s fattore di scala in x,y
// Output:
// surf oggetto che rappresenta la superficie
// ritorna 0 se c' stato un errore

#ifdef __cplusplus
}
#endif

#endif