#include "glui.h"
#include <malloc.h>
#include <math.h>
#include "snurbs.h"
#include "cs_surface.h"

//*** Definitions

#define EXTRUSION		100
#define RULED			101
#define ROTATION		102
#define SKINNING		103
#define SWINGING		104

#define START			200
#define GENERATE		201
#define ORDERU			202
#define ORDERV			203
#define CURVE			204
#define CHOICE			205
#define SK_DIST			206
#define SAVE			207
#define SAVE_PATH		208
#define CLOSECURVE		209
#define SHOWALL			210
#define MODIFY			211
#define QUIT			212

#define KNOT_ELEM		1000
#define WEIGHT_ELEM		2000

#define CURVE_MODE		300
#define SURF_MODE		301

#define MAX_ORDER		8
#define MAX_NCP			20
#define MAX_SECTIONS	10
#define DIMGRID			5

void GLUI_handler(int button_id);
void add_points(int x, int y, int ci, int ncp);
int selection(int x, int y, int ncp, GLfloat d, int ci);
void load_weight(int ci, int i);
void delete_point(int i, int ncp, int ci);
void init_weight(int ncp);
void update_knot_vector(int order, int ncp);
void load_knot_vector(int ci, int order, int ncp);
void load_weight_vector(int ci, int ncp);


//*** Global variables

GLfloat light0_amb[] = {.0f,.0f,.0f,1.0f};
GLfloat light0_diff[] = {1.0f,1.0f,1.0f,1.0f};
GLfloat light0_spec[] = {1.0f,1.0f,1.0f,1.0f};
GLfloat light0_pos[] = {2.0f,2.0f,-3.0f,1.0f};

GLfloat light1_amb[] = {.0f,.0f,.0f,1.0f};
GLfloat light1_diff[] = {.5f,0.0f,0.0f,1.0f};
GLfloat light1_spec[] = {1.0f,1.0f,1.0f,1.0f};
GLfloat light1_pos[] = {-2.0f,-2.0f,-3.0f,1.0f};

GLfloat mat0_amb[] = {1.0f,1.0f,1.0f,1.0f};
GLfloat mat0_diff[] = {.5f,.5f,.5f,1.0f};
GLfloat mat0_spec[] = {1.0f,1.0f,1.0f,1.0f};
GLfloat mat0_shine[] = {20.0f};

int display_mode = SURF_MODE;	//Sceglie tra visualizzazione curve e superfici

int main_win;	//finestra principale

int move_point = 0;
int point = 0;
int last_x = 0;
int last_y = 0;
GLfloat position[2];

GLfloat max_x = DIMGRID,min_x = -DIMGRID,max_y = DIMGRID,min_y = -DIMGRID; //estremi finestra

//valori correnti delle curve
struct cnurbs curve[MAX_SECTIONS];
GLfloat CPtemp[MAX_NCP*4];
int ncurve;
int type;
int isclosed;
int dim;
int ncp;
GLfloat distance[MAX_SECTIONS];

//superficie generata
struct snurbs surf;
GLuint surflist=0;
GLUnurbsObj* sobj;

//*** Global Controls

GLUI_Checkbox* rational;
GLUI* option_win;
GLUI* foot_panel;
GLUI* right_panel;
GLUI_Spinner* orderu;
GLUI_Spinner* orderv;
GLUI_EditText* knots[MAX_ORDER+MAX_NCP];
GLUI_Rollout* knot;
GLUI_EditText* weights[MAX_NCP];
GLUI_Rollout* weight;
GLUI_Rollout* dist_roll;
GLUI_Spinner* dist[3];
GLUI_Spinner* cross_curve;
GLUI_Listbox* choice;
GLUI_Spinner* num_curve;
GLUI_EditText* sk_dist;
GLUI_Button* save_obj;
GLUI_EditText* save_path;
GLUI_Checkbox* showall;
GLUI_Button* modify;
GLUI_Checkbox* closed;

//*** Live variables

GLfloat tranXY[2] = {0.0f,0.0f};
GLfloat tranZ[1] = {0.0f};
GLfloat rot[16];

//*** debug
//*** salva in un file di testo i valori correnti dei vettori curve[] e cpoints[]
void print_vectors()
{
	FILE* debug;
	int i,j;
	if ((debug = fopen("debug.txt","w+"))==NULL)
		exit(1);
	fprintf(debug,"\n--------------------------------------------------------------\n");
	if (dim == 4)
		for (i=0;i<ncurve;i++)
		{
			fprintf(debug,"curve[%d].order = %d\n",i,curve[i].order);
			fprintf(debug,"curve[%d].ncp = %d\n",i,curve[i].ncp);
			fprintf(debug,"curve[%d].nk = %d\n",i,curve[i].nk);
			for (j=0;j<curve[i].nk;j++)
                fprintf(debug,"curve[%d].K[%d] = %f\n",i,j,curve[i].K[j]);
			for (j=0;j<curve[i].ncp;j++)
				fprintf(debug,"curve[%d][%d].CP = %f %f %f %f\n",i,j,curve[i].CP[4*j],curve[i].CP[4*j+1],curve[i].CP[4*j+2],curve[i].CP[4*j+3]);
		}
	else
		for (i=0;i<ncurve;i++)
		{
			fprintf(debug,"curve[%d].order = %d\n",i,curve[i].order);
			fprintf(debug,"curve[%d].ncp = %d\n",i,curve[i].ncp);
			fprintf(debug,"curve[%d].nk = %d\n",i,curve[i].nk);
			for (j=0;j<curve[i].nk;j++)
                fprintf(debug,"curve[%d].K[%d] = %f\n",i,j,curve[i].K[j]);
			for (j=0;j<curve[i].ncp;j++)
				fprintf(debug,"curve[%d][%d].CP = %f %f %f\n",i,j,curve[i].CP[3*j],curve[i].CP[3*j+1],curve[i].CP[3*j+2]);
		}
	fclose(debug);
}

//*** window constructors

void create_curve_window()
// crea la finestra di opzione per le curve sezione
{
	int i;
	option_win = GLUI_Master.create_glui("Opzioni curve sezione",0,500,500);
	orderu = option_win->add_spinner("order u",GLUI_SPINNER_INT,NULL,ORDERU,GLUI_handler);
	orderu->set_int_limits(2,MAX_ORDER);
	orderu->set_speed(.1f);
	knot = option_win->add_rollout("knots",0);
	char label[10];
	for ( i=0;i<MAX_ORDER+MAX_NCP;i++)
	{
		itoa(i+1,label,10);
		knots[i] = option_win->add_edittext_to_panel(knot,label,GLUI_EDITTEXT_FLOAT,NULL,KNOT_ELEM+i,GLUI_handler);
		knots[i]->set_float_limits(0,1);
	}
	cross_curve = option_win->add_spinner("curva",GLUI_SPINNER_INT,NULL,CURVE,GLUI_handler);
	cross_curve->set_int_limits(1,MAX_SECTIONS);
	cross_curve->set_speed(.1f);
	sk_dist = option_win->add_edittext("distanza",GLUI_EDITTEXT_FLOAT,NULL,SK_DIST,GLUI_handler);
	sk_dist->set_float_limits(0.0f,5.0f);
	option_win->add_column(0);
	orderv = option_win->add_spinner("order v",GLUI_SPINNER_INT,NULL,ORDERV,GLUI_handler);
	orderv->set_int_limits(2,MAX_ORDER);
	orderv->set_speed(.1f);
	weight = option_win->add_rollout("weights",0);
	for (i=0;i<MAX_NCP;i++)
	{
		itoa(i+1,label,10);
		weights[i] = option_win->add_edittext_to_panel(weight,label,GLUI_EDITTEXT_FLOAT,NULL,WEIGHT_ELEM+i,GLUI_handler);
		weights[i]->set_float_limits(0,10);
	}
	dist_roll = option_win->add_rollout("Vettore Distanza",0);
	char* dist_label[] = {"x:","y:","z:"};
	for (i=0;i<3;i++)
	{
		dist[i] = option_win->add_spinner_to_panel(dist_roll,dist_label[i],GLUI_SPINNER_FLOAT,NULL);
		dist[i]->set_float_limits(0.0f,5.0f);
		dist[i]->set_speed(.1f);
	}
	showall = option_win->add_checkbox("Mostra curve",NULL,SHOWALL,GLUI_handler);
	GLUI_Button* ok = option_win->add_button("Mostra superficie",GENERATE,GLUI_handler);
}

//*** Mouse callback (gestisce i click del mouse)
void mouseFunc(int button, int state, int x, int y)
{
	int i,j,k;
	int ci = cross_curve->get_int_val()-1;
	if (display_mode == CURVE_MODE)		//stiamo generando le curve sezione
	{
		if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN))	//click tasto sinistro (aggiunta)
			{
				if (type==SWINGING)		//nel caso di swinging
				{
					if ((i = selection(x,y,curve[ci].ncp,.1f,ci))==-1)
					// se non viene cliccato un cpoint ne aggiunge uno
					{
						if (curve[ci].ncp < MAX_NCP)
						{
							if (ci==0)
								add_points(x,y,ci,curve[ci].ncp);
							else
							{
								add_points(x,y,ci,curve[ci].ncp-1);
								for (j=0;j<dim;j++)
									curve[ci].CP[curve[ci].dim*(curve[ci].ncp-1)+j] =
									curve[ci].CP[j];
							}

							update_knot_vector(orderu->get_int_val(),curve[ci].ncp);

							if (dim==4)
								init_weight(curve[ci].ncp);

							load_knot_vector(ci,orderu->get_int_val(),curve[ci].ncp);
							load_weight_vector(ci,curve[ci].ncp);
							glutSetWindow(main_win);
							glutPostRedisplay();
						}
					}
					else	//altrimenti muove il control point
					{
						move_point = 1;
						point = i;
						last_x = x;
						last_y = y;
						position[0] = curve[ci].CP[curve[ci].dim*point];
						position[1] = curve[ci].CP[curve[ci].dim*point+1];
					}
				}
				else if (!isclosed)	//se vogliamo delle curve sezione aperte
				{
					if ((i = selection(x,y,ncp,.1f,ci))==-1)
					{
						if (ncp < MAX_NCP)
						{
							for (j=0;j<ncurve;j++)
								add_points(x,y,j,ncp);
							ncp++;
							update_knot_vector(orderu->get_int_val(),ncp);

							if (dim == 4)
								init_weight(ncp);

							for (j=0;j<ncurve;j++)
							{
								load_knot_vector(j,orderu->get_int_val(),ncp);
								load_weight_vector(j,ncp);
							}
							glutSetWindow(main_win);
							glutPostRedisplay();
						}
					}
					else
					{
						move_point = 1;
						point = i;
						last_x = x;
						last_y = y;
						position[0] = curve[ci].CP[curve[ci].dim*point];
						position[1] = curve[ci].CP[curve[ci].dim*point+1];
					}
				}
				else		//nel caso di curve sezione chiuse
				{
					if ((i = selection(x,y,ncp,.1f,ci))==-1)
					{
						if (ncp < MAX_NCP)
						{
							for (k=0;k<ncurve;k++)
							{
								add_points(x,y,k,ncp-1);
								for (j=0;j<dim;j++)
									curve[k].CP[curve[k].dim*ncp+j] = curve[k].CP[j];
							}

							ncp++;
                            update_knot_vector(orderu->get_int_val(),ncp);

							if (dim==4)
								init_weight(ncp);

							for (j=0;j<ncurve;j++)
								{
									load_knot_vector(j,orderu->get_int_val(),ncp);
									load_weight_vector(j,ncp);
								}
							glutSetWindow(main_win);
							glutPostRedisplay();
						}
					}
					else
					{
						move_point = 1;
						point = i;
						last_x = x;
						last_y = y;
						position[0] = curve[ci].CP[curve[ci].dim*point];
						position[1] = curve[ci].CP[curve[ci].dim*point+1];
					}
				}
			}

		if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_UP))	// rilascio del tasto sinistro
			move_point = 0;

		if ((button == GLUT_RIGHT_BUTTON) && (state == GLUT_DOWN))	// click del tasto destro (cancellazione)
		{
			int order;
			if (type==SWINGING)	// nel caso di swinging
			{
				i = selection(x,y,curve[ci].ncp,.1f,ci);
				if (((ci==1)&&(curve[ci].ncp>4)&&(i!=-1)&&(i != 0)&&(i != curve[ci].ncp-1))
					|| ((ci==0)&&(curve[ci].ncp>2)&&(i!=-1)))
					{
						delete_point(i,curve[ci].ncp,ci);
						curve[ci].ncp--;
						if (orderu->get_int_val() > curve[ci].ncp)
							orderu->set_int_val(curve[ci].ncp);
						order = orderu->get_int_val();
						update_knot_vector(order,curve[ci].ncp);
						init_weight(curve[ci].ncp);
						curve[ci].order = order;
						load_knot_vector(ci,order,curve[ci].ncp);
						load_weight_vector(ci,curve[ci].ncp);

						glutSetWindow(main_win);
						glutPostRedisplay();
					}
			}
			else if (!isclosed)	// nel caso di curve aperte
			{
				if ((ncp>2)&&((i = selection(x,y,ncp,.1f,ci))!=-1))
				{
					ncp--;
					if ( orderu->get_int_val() > ncp)
						orderu->set_int_val(ncp);
					order = orderu->get_int_val();
					update_knot_vector(order,ncp);
					init_weight(ncp);
					for (j=0;j<ncurve;j++)
						{
						delete_point(i,ncp+1,j);
						curve[j].ncp--;
						curve[j].order = order;
						load_knot_vector(j,order,ncp);
						load_weight_vector(j,ncp);
					}
					glutSetWindow(main_win);
					glutPostRedisplay();
				}
			}
			else		// caso di curve chiuse
			{
				i = selection(x,y,ncp,.1f,ci);
				if ((ncp>4)&&(i!=-1)&&(i != 0)&&(i != ncp-1))
				{
					ncp--;
					if ( orderu->get_int_val() > ncp)
						orderu->set_int_val(ncp);
					order = orderu->get_int_val();
					update_knot_vector(order,ncp);
					init_weight(ncp);
					for (j=0;j<ncurve;j++)
						{
						delete_point(i,ncp+1,j);
						curve[j].ncp--;
						curve[j].order = order;
						load_knot_vector(j,order,ncp);
						load_weight_vector(j,ncp);
					}
					glutSetWindow(main_win);
					glutPostRedisplay();
				}
			}

		}
	}

}

//*** gestione del movimento del mouse
void motionFunc(int x, int y)
{
	int tx, ty, tw, th;
	int X,Y;
	
	if (move_point == 1)	//il trascinamento del tasto sinistro su un cp ne causa lo spostamento
	{
		int ci = cross_curve->get_int_val()-1;
		GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
		X = x - last_x;
		Y = y - last_y;
		position[0] += DIMGRID*(2*(GLfloat)X/tw);
		position[1] += -DIMGRID*(2*(GLfloat)Y/th);
		if (position[0] > max_x)	//test per evitare che il cp esca dalla finestra
			curve[ci].CP[curve[ci].dim*point] = max_x;
		else if (position[0] < min_x)
			curve[ci].CP[curve[ci].dim*point] = min_x;
		else
			curve[ci].CP[curve[ci].dim*point] = position[0];
		if (position[1] > max_y)
			curve[ci].CP[curve[ci].dim*point+1] = max_y;
		else if (position[1] < min_y)
			curve[ci].CP[curve[ci].dim*point+1] = min_y;
		else
			curve[ci].CP[curve[ci].dim*point+1] = position[1];
		last_x = x;
		last_y = y;

		if ((type==SWINGING)&&(ci==1))	//nel caso di curve chiuse muove assieme il primo e l'ultimo cp
			if (point == 0)
			{
				curve[1].CP[curve[1].dim*(curve[1].ncp-1)] = curve[1].CP[0];
				curve[1].CP[curve[1].dim*(curve[1].ncp-1)+1] = curve[1].CP[1];
				load_weight(1,curve[1].ncp-1);
			}
		if (isclosed && (point == 0))
		{

			curve[ci].CP[curve[ci].dim*(curve[ci].ncp-1)] = curve[ci].CP[0];
			curve[ci].CP[curve[ci].dim*(curve[ci].ncp-1)+1] = curve[ci].CP[1];
			load_weight(ci,curve[ci].ncp-1);
		}
 
		load_weight(ci,point);
		last_x = x;
		last_y = y;
		glutSetWindow(main_win);
		glutPostRedisplay();
	}
}

//***aggiorna la vista dei nodi in base all'ordine e al numero di cp della curva
void update_knot_vector(int order, int ncp)
{
	int nk = order+ncp;
	int span = nk - 2*order + 1;
	for (int i=0;i<MAX_NCP+MAX_ORDER;i++)
	{
		if (i<order)
		{
			knots[i]->enable();
			knots[i]->set_float_val(0.0f);
		}
		else if ((i >= (nk-order)) && (i < nk))
		{
			knots[i]->enable();
			knots[i]->set_float_val(1.0f);
		}
		else if (i >= nk)
		{
			knots[i]->disable();
			knots[i]->set_float_val(0);
		}
		else
		{
			knots[i]->enable();
			knots[i]->set_float_val(1/(GLfloat)span*(i-order+1));
		}
	}
}

//*** aggiorna la vista dei pesi in base al numero di cp della curva
void update_weights_vector(int ncp)
{
	for (int i=0;i<MAX_NCP;i++)
	{
		if (i<ncp)
		{
			if (!weights[i]->enabled)
			{
				weights[i]->enable();
				weights[i]->set_float_val(1.0f);
			}
		}
		else
		{
			weights[i]->disable();
			weights[i]->set_float_val(0.0f);
		}
	}
}

//*** azzera la vista dei pesi in base al numero di cp della curva
void init_weight(int ncp)
{
	for (int i=0;i<MAX_NCP;i++)
	{
		if (i<ncp)
		{
			weights[i]->enable();
			weights[i]->set_float_val(1.0f);
		}
		else
		{
			weights[i]->disable();
			weights[i]->set_float_val(0.0f);
		}
	}
}

//*** aggiorna la vista dei nodi mantenendone l'ordine non decrescente
//*** i  il nodo modificato, order l'ordine della curva e ncp il numero dei suoi cp
void change_knot(int i, int order, int ncp)
{
	int nk = order+ncp;
	if (i<order)
		knots[i]->set_float_val(0.0f);
	else if (i >= (nk-order))
		knots[i]->set_float_val(1.0f);
	else
	{
		if (knots[i]->get_float_val() < knots[i-1]->get_float_val())
			knots[i]->set_float_val(knots[i-1]->get_float_val());
		else if (knots[i]->get_float_val() > knots[i+1]->get_float_val())
			knots[i]->set_float_val(knots[i+1]->get_float_val());
	}
}
//*** aggiorna il vettore nodale delle curve basandosi sui valori della vista
void load_knot_vector(int ci, int order, int ncp)
{
	int nk = order+ncp;
	for (int i=0;i<nk;i++)
		curve[ci].K[i] = knots[i]->get_float_val();
	curve[ci].nk = nk;
}

//*** aggiorna il vettore dei cp delle curve basandosi sui valori dei pesi nella vista
void load_weight_vector(int ci, int ncp)
{
	GLfloat w;
	if (dim == 4)
	{
		for (int i=0;i<ncp;i++)
		{
			w = weights[i]->get_float_val();
			curve[ci].CP[4*i+3] = w;
		}
	}
}

//*** aggiorna la vista dei pesi basandosi sul vettore dei cp della curva ci
void reload_weights(int ci, int ncp)
{
	int i;
	for (i=0;i<ncp;i++)
	{
		weights[i]->enable();
		weights[i]->set_float_val(curve[ci].CP[4*i+3]);
	}
	for (i=ncp;i<MAX_NCP;i++)
	{
		weights[i]->disable();
		weights[i]->set_float_val(0.0f);
	}
}

//*** aggiorna la vista dei nodi basandosi sul vettore nodale della curva ci
void reload_knots(int ci, int order, int ncp)
{
	int i;
	int nk = order + ncp;
	for (i=0;i<nk;i++)
	{
		knots[i]->enable();
		knots[i]->set_float_val(curve[ci].K[i]);
	}
	for (i=nk;i<MAX_NCP+MAX_ORDER;i++)
	{
		knots[i]->disable();
		knots[i]->set_float_val(0.0f);
	}
}

//*** aggiorna il peso i-esimo della curva ci basandosi sulla vista dei pesi
void load_weight(int ci, int i)
{
	GLfloat w;
	if (dim == 4)
	{
		w = weights[i]->get_float_val();
		curve[ci].CP[4*i+3] = w;
	}
}

//*** sceglie il tipo di proiezione (prospettica per la visione della superficie
//*** e ortografica per l'inserimento delle curve sezione
void update_projection()
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	glViewport( tx, ty, tw, th );

	GLfloat xy_aspect = (GLfloat)tw / (GLfloat)th;

	switch (display_mode)
	{
	case SURF_MODE:
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60,xy_aspect,1,20);
		glMatrixMode(GL_MODELVIEW);
		break;
	case CURVE_MODE:
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(-DIMGRID,DIMGRID,-DIMGRID,DIMGRID);
		glMatrixMode(GL_MODELVIEW);
		break;
	default:
		exit(1);
	}
}

//*** disegna il punto di controllo p con lato pari a 2*dim
void draw_point(GLfloat* p, GLfloat dim)
{
	glBegin(GL_QUADS);
		glVertex2f(p[0]-dim,p[1]+dim);
		glVertex2f(p[0]+dim,p[1]+dim);
		glVertex2f(p[0]+dim,p[1]-dim);
		glVertex2f(p[0]-dim,p[1]-dim);
	glEnd();
}

//*** disegna il poligono di controllo (ci) e la corrispondente curva sezione
void draw_curve(int ci)
{
	GLenum map;
	int i,j;

	if (ci==cross_curve->get_int_val()-1)
		glColor3f(1.0f,0.0f,0.0f);
	else
		glColor3f(.5f,.5f,.5f);
	GLfloat* pt = (GLfloat*)malloc(curve[ci].dim*curve[ci].ncp*sizeof(GLfloat));
	if (curve[ci].dim == 3)
	{
		for (i=0;i<curve[ci].ncp;i++)
			draw_point(&(curve[ci].CP[3*i]),.1f);
		glBegin(GL_LINE_STRIP);
		for (i=0;i<curve[ci].ncp;i++)
			glVertex2f(curve[ci].CP[3*i],curve[ci].CP[3*i+1]);
		glEnd();
		if (ci==cross_curve->get_int_val()-1)
			glColor3f(0.0f,1.0f,0.0f);
		else
			glColor3f(.5f,.5f,.5f);
		map = GL_MAP1_VERTEX_3;
		for (i=0;i<curve[ci].ncp;i++)
			for (j=0;j<3;j++)
				pt[3*i+j] = curve[ci].CP[3*i+j];
	}
	else
	{
		for (i=0;i<curve[ci].ncp;i++)
			draw_point(&(curve[ci].CP[4*i]),.1f);
		glBegin(GL_LINE_STRIP);
		for (i=0;i<curve[ci].ncp;i++)
			glVertex2f(curve[ci].CP[4*i],curve[ci].CP[4*i+1]);
		glEnd();
		if (ci==cross_curve->get_int_val()-1)
			glColor3f(0.0f,1.0f,0.0f);
		else
			glColor3f(.5f,.5f,.5f);
		map = GL_MAP1_VERTEX_4;
		for (i=0;i<curve[ci].ncp;i++)
		{
			for (j=0;j<3;j++)
				pt[4*i+j] = curve[ci].CP[4*i+j]*curve[ci].CP[4*i+3];
			pt[4*i+3] = curve[ci].CP[4*i+3];
		}
	}

	GLUnurbsObj* cobj = gluNewNurbsRenderer();
	gluNurbsProperty(cobj,GLU_SAMPLING_TOLERANCE,20);
	gluBeginCurve(cobj);
		gluNurbsCurve(cobj,curve[ci].nk,curve[ci].K,curve[ci].dim,pt,curve[ci].order,map);
	gluEndCurve(cobj);
	gluDeleteNurbsRenderer(cobj);
	free(pt);
}

//*** aggiunge il cp alla curva ci basandosi sulla posizione del mouse al momento del click
void add_points(int x, int y, int ci, int ncp)
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	curve[ci].CP[curve[ci].dim*ncp] = DIMGRID*(2*(GLfloat)x/tw-1);
	curve[ci].CP[curve[ci].dim*ncp+1] = -DIMGRID*(2*(GLfloat)y/th-1);
	curve[ci].CP[curve[ci].dim*ncp+2] = 0.0f;
	curve[ci].ncp++;
}

//*** cancella l'i-esimo cp della curva ci
void delete_point(int i, int ncp, int ci)
{
	if ((ncp>2)&&(i<ncp-1))
	{
		for (int j=i;j<ncp-1;j++)
		{
			curve[ci].CP[curve[ci].dim*j] = curve[ci].CP[curve[ci].dim*(j+1)];
			curve[ci].CP[curve[ci].dim*j+1] = curve[ci].CP[curve[ci].dim*(j+1)+1];
			curve[ci].CP[curve[ci].dim*j+2] = curve[ci].CP[curve[ci].dim*(j+1)+2];
		}
	}
}

//*** disegna la griglia per il riferimento
void draw_grid()
{
	int i;
	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_LINES);
		glVertex2f(-DIMGRID,0.0f);
		glVertex2f(DIMGRID,0.0f);
		glVertex2f(0.0f,DIMGRID);
		glVertex2f(0.0f,-DIMGRID);
		for (i=-DIMGRID+1;i<DIMGRID;i++)
		{
			glVertex2f((GLfloat)i,.1f);
			glVertex2f((GLfloat)i,-.1f);
		}
		for (i=-DIMGRID+1;i<DIMGRID;i++)
		{
			glVertex2f(.1f,(GLfloat)i);
			glVertex2f(-.1f,(GLfloat)i);
		}
	glEnd();
}

//*** stabilisce se un punto  stato selezionato in base alla posizione del mouse al momento del click
//*** restituisce la posizione del cp nella curva ci oppure -1 se nessun cp  stato selezionato
int selection(int x, int y, int ncp, GLfloat d, int ci)
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	GLfloat X = DIMGRID*(2*(GLfloat)x/tw-1);
	GLfloat Y = -DIMGRID*(2*(GLfloat)y/th-1);
	for (int i=0;i<ncp;i++)
	{
		if ((X >= curve[ci].CP[curve[ci].dim*i] - d) && (X <= curve[ci].CP[curve[ci].dim*i] + d)
			&& (Y >= curve[ci].CP[curve[ci].dim*i+1] - d) && (Y <= curve[ci].CP[curve[ci].dim*i+1] + d))
			return i;
	}
	return -1;
}

//*** GLUI Callbacks

void GLUI_handler(int id)
{
int i,j,k;
	int curvei = cross_curve->get_int_val()-1;
	GLfloat skd;
	GLfloat rad2=1/sqrt(2.);
	GLfloat cp[9][4]={{1,0,0,1},{1,1,0,rad2},{0,1,0,1},{-1,1,0,rad2},{-1,0,0,1},{-1,-1,0,rad2},{0,-1,0,1},{1,-1,0,rad2},{1,0,0,1}};
	GLfloat cp2[9][3]={{1,0,0},{1,1,0},{0,1,0},{-1,1,0},{-1,0,0},{-1,-1,0},{0,-1,0},{1,-1,0},{1,0,0}};
	GLfloat knots_rot[12]={0,0,0,.25,.25,.5,.5,.75,.75,1,1,1};
	char* name = NULL;
	char nomefile[30];
	int length;
	switch (id)		//identifica quale controllo ha generato l'evento
	{
	case MODIFY:	//modifica dei parametri della superficie correntemente visualizzata
		foot_panel->disable();
		right_panel->disable();

		display_mode = CURVE_MODE;

		if ( glutGetWindow() != main_win ) 
			glutSetWindow(main_win);
		update_projection();
		glutPostRedisplay();

		option_win->show();
		break;

	case START:		//inizia la generazione di una nuova superficie passando all'inserimento delle curve sezione
		foot_panel->disable();
		right_panel->disable();
		if (rational->get_int_val())
			dim=4;
		else
			dim=3;
		ncp = 2;
		type = choice->get_int_val();
		orderu->set_int_val(2);
		orderv->set_int_val(2);
		isclosed = closed->get_int_val();
	
		showall->set_int_val(false);

		//Free memory
		for (i=0;i<ncurve;i++)
		{
			CNURBS_FREE(curve[i]);
		}

		SNURBS_FREE(surf);

		switch (type)		//inizializza le curve sezione in base al tipo di operazione richiesto
		{
		case EXTRUSION:
			showall->disable();
			orderu->enable();
			orderv->disable();
			knot->enable();
			weight->enable();
			cross_curve->disable();
			dist_roll->enable();
			sk_dist->disable();
			ncurve=1;
			cross_curve->set_int_limits(1,1);
			if (!isclosed)
			{
				curve[0].dim = dim;
				curve[0].order = orderu->get_int_val();
				curve[0].ncp = 2;
				curve[0].nk = curve[0].order + curve[0].ncp;
				curve[0].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
				curve[0].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));
				curve[0].CP[0] = -4.0f;
				curve[0].CP[1] = 0.0f;
				curve[0].CP[2] = 0.0f;
				curve[0].CP[curve[0].dim] = -3.0f;
				curve[0].CP[curve[0].dim+1] = 0.0f;
				curve[0].CP[curve[0].dim+2] = 0.0f;
				if (dim == 4)
				{
					curve[0].CP[3] = 1.0f;
					curve[0].CP[7] = 1.0f;
				}
				curve[0].K[0] = 0.0f;
				curve[0].K[1] = 0.0f;
				curve[0].K[2] = 1.0f;
				curve[0].K[3] = 1.0f;
			}
			else
			{
				ncp = 9;
				orderu->set_int_val(3);
				curve[0].dim = dim;
				curve[0].order = 3;
				curve[0].ncp = 9;
				curve[0].nk = curve[0].order + curve[0].ncp;
				curve[0].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
				curve[0].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));

				if (dim==3)
					for (i=0;i<9;i++)
						for (j=0;j<3;j++)
							curve[0].CP[3*i+j] = cp2[i][j];
				else
					for (i=0;i<9;i++)
						for (j=0;j<4;j++)
							curve[0].CP[4*i+j] = cp[i][j];
			
				for (i=0;i<12;i++)
					curve[0].K[i] = knots_rot[i];
			}

			max_x = DIMGRID;

			break;
		case RULED:
			showall->enable();
			orderu->enable();
			orderv->disable();
			knot->enable();
			weight->enable();
			cross_curve->enable();
			dist_roll->enable();
			sk_dist->disable();
			ncurve=2;
			cross_curve->set_int_limits(1,2);
			if (!isclosed)
			{
				for (k=0; k<ncurve; k++)
				{
					curve[k].dim = dim;
					curve[k].order = orderu->get_int_val();
					curve[k].ncp = 2;
					curve[k].nk = curve[k].order + curve[k].ncp;
					curve[k].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
					curve[k].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));
					curve[k].CP[0] = -4.0f;
					curve[k].CP[1] = 0.0f;
					curve[k].CP[2] = 0.0f;
					curve[k].CP[curve[k].dim] = -3.0f;
					curve[k].CP[curve[k].dim+1] = 0.0f;
					curve[k].CP[curve[k].dim+2] = 0.0f;
					if (dim == 4)
					{
						curve[k].CP[3] = 1.0f;
						curve[k].CP[7] = 1.0f;
					}
					curve[k].K[0] = 0.0f;
					curve[k].K[1] = 0.0f;
					curve[k].K[2] = 1.0f;
					curve[k].K[3] = 1.0f;
				}
			}
			else
			{
				ncp = 9;
				orderu->set_int_val(3);
				for (k=0;k<ncurve;k++)
				{
					curve[k].dim = dim;
					curve[k].order = 3;
					curve[k].ncp = 9;
					curve[k].nk = curve[k].order + curve[k].ncp;
					curve[k].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
					curve[k].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));

					if (dim==3)
						for (i=0;i<9;i++)
							for (j=0;j<3;j++)
								curve[k].CP[3*i+j] = cp2[i][j];
					else
						for (i=0;i<9;i++)
							for (j=0;j<4;j++)
								curve[k].CP[4*i+j] = cp[i][j];
			
					for (i=0;i<12;i++)
						curve[k].K[i] = knots_rot[i];
				}
			}

			max_x = DIMGRID;
			break;
		case ROTATION:
			showall->disable();
			orderu->enable();
			orderv->disable();
			knot->enable();
			weight->enable();
			cross_curve->disable();
			cross_curve->set_int_limits(1,1);
			dist_roll->disable();
			sk_dist->disable();
			ncurve=1;

			curve[0].dim = dim;
			curve[0].order = orderu->get_int_val();
			curve[0].ncp = 2;
			curve[0].nk = curve[0].order + curve[0].ncp;
			curve[0].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
			curve[0].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));
			curve[0].CP[0] = -4.0f;
			curve[0].CP[1] = 0.0f;
			curve[0].CP[2] = 0.0f;
			curve[0].CP[curve[0].dim] = -3.0f;
			curve[0].CP[curve[0].dim+1] = 0.0f;
			curve[0].CP[curve[0].dim+2] = 0.0f;
			if (dim == 4)
			{
				curve[0].CP[3] = 1.0f;
				curve[0].CP[7] = 1.0f;
			}
			curve[0].K[0] = 0.0f;
			curve[0].K[1] = 0.0f;
			curve[0].K[2] = 1.0f;
			curve[0].K[3] = 1.0f;

			max_x = 0;
			break;
		case SWINGING:
			showall->disable();
			orderu->enable();
			orderv->disable();
			knot->enable();
			weight->enable();
			cross_curve->enable();
			cross_curve->set_int_limits(1,2);
			dist_roll->disable();
			sk_dist->disable();
			ncurve=2;

			//curva profilo
			curve[0].dim = dim;
			curve[0].order = orderu->get_int_val();
			curve[0].ncp = 2;
			curve[0].nk = curve[0].order + curve[0].ncp;
			curve[0].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
			curve[0].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));
			curve[0].CP[0] = -4.0f;
			curve[0].CP[1] = 0.0f;
			curve[0].CP[2] = 0.0f;
			curve[0].CP[curve[0].dim] = -3.0f;
			curve[0].CP[curve[0].dim+1] = 0.0f;
			curve[0].CP[curve[0].dim+2] = 0.0f;
			if (dim == 4)
			{
				curve[0].CP[3] = 1.0f;
				curve[0].CP[7] = 1.0f;
			}
			curve[0].K[0] = 0.0f;
			curve[0].K[1] = 0.0f;
			curve[0].K[2] = 1.0f;
			curve[0].K[3] = 1.0f;

			//curva traiettoria
			curve[1].dim = dim;
			curve[1].order = 3;
			curve[1].ncp = 9;
			curve[1].nk = curve[1].order + curve[1].ncp;
			curve[1].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
			curve[1].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));

			if (dim==3)
				for (i=0;i<9;i++)
					for (j=0;j<3;j++)
						curve[1].CP[3*i+j] = cp2[i][j];
			else
				for (i=0;i<9;i++)
					for (j=0;j<4;j++)
						curve[1].CP[4*i+j] = cp[i][j];
			
			for (i=0;i<12;i++)
				curve[1].K[i] = knots_rot[i];

			max_x = 0;
			break;
		case SKINNING:
			showall->enable();
			orderu->enable();
			orderv->enable();
			knot->enable();
			weight->enable();
			cross_curve->enable();
			dist_roll->disable();
			ncurve=num_curve->get_int_val();
			cross_curve->set_int_limits(1,ncurve);
			sk_dist->enable();
			orderv->set_int_limits(2,ncurve);

			if (!isclosed)
			{
				for (k=0; k<ncurve; k++)
				{
					curve[k].dim = dim;
					curve[k].order = orderu->get_int_val();
					curve[k].ncp = 2;
					curve[k].nk = curve[k].order + curve[k].ncp;
					curve[k].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
					curve[k].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));
					curve[k].CP[0] = -4.0f;
					curve[k].CP[1] = 0.0f;
					curve[k].CP[2] = 0.0f;
					curve[k].CP[curve[k].dim] = -3.0f;
					curve[k].CP[curve[k].dim+1] = 0.0f;
					curve[k].CP[curve[k].dim+2] = 0.0f;
					if (dim == 4)
					{
						curve[k].CP[3] = 1.0f;
						curve[k].CP[7] = 1.0f;
					}
					curve[k].K[0] = 0.0f;
					curve[k].K[1] = 0.0f;
					curve[k].K[2] = 1.0f;
					curve[k].K[3] = 1.0f;
				}
			}
			else
			{
				ncp = 9;
				orderu->set_int_val(3);
				for (k=0;k<ncurve;k++)
				{
					curve[k].dim = dim;
					curve[k].order = 3;
					curve[k].ncp = 9;
					curve[k].nk = curve[k].order + curve[k].ncp;
					curve[k].K = (GLfloat*)malloc((MAX_NCP+MAX_ORDER)*sizeof(GLfloat));
					curve[k].CP = (GLfloat*)malloc(MAX_NCP*dim*sizeof(GLfloat));

					if (dim==3)
						for (i=0;i<9;i++)
							for (j=0;j<3;j++)
								curve[k].CP[3*i+j] = cp2[i][j];
					else
						for (i=0;i<9;i++)
							for (j=0;j<4;j++)
								curve[k].CP[4*i+j] = cp[i][j];
			
					for (i=0;i<12;i++)
						curve[k].K[i] = knots_rot[i];
				}
			}
			for (i=0;i<ncurve;i++)
				distance[i] = (GLfloat)i/2;
			sk_dist->set_float_val(0.0f);

			max_x = DIMGRID;
			break;
		default:
			break;
		}
		orderu->enable();
		knot->enable();
		if (dim==3)
			weight->disable();
		else
			weight->enable();

		num_curve->set_int_val(1);
		cross_curve->set_int_val(1);
		knot->close();
		weight->close();
		dist_roll->close();
		if (!isclosed)
		{
			update_knot_vector(orderu->get_int_val(),ncp);
			init_weight(ncp);
		}
		else
		{
			reload_weights(0,ncp);
			reload_knots(0,orderu->get_int_val(),ncp);
		}
		dist[0]->set_float_val(0.0f);
		dist[1]->set_float_val(0.0f);
		dist[2]->set_float_val(1.0f);

		display_mode = CURVE_MODE;

		if ( glutGetWindow() != main_win ) 
			glutSetWindow(main_win);
		update_projection();
		glutPostRedisplay();

		option_win->show();
		break;

	case SAVE_PATH:		//controlla la correttezza del nome inserito per il salvataggio in formato snurbs
		name = save_path->get_text();
		length = (int)strlen(name);
		if ((length>8)||(length==0))
            save_obj->disable();
		else
		{
			int test = 1;
			for (i=0;i<length;i++)
			{
				if (!(((name[i]>='a')&&(name[i]<='z'))||((name[i]>='A')&&(name[i]<='Z'))||((name[i]>='0')&&(name[i]<='9'))))
				{
				test = 0;
				break;
				}
			}
			if (test)
				save_obj->enable();
			else
				save_obj->disable();
		}

		break;

	case SAVE:		//salva la superficie visualizzata in formato snurbs
		strcpy(nomefile,save_path->get_text());
		strcat(nomefile,".snurbs");
		writeSnurbs(nomefile,surf);
		save_path->disable();
		save_obj->disable();
		break;

	case GENERATE:		//genera la superficie basandosi sulle curve sezione inserite
		option_win->hide();
		foot_panel->enable();
		right_panel->enable();

		if (choice->get_int_val()==SKINNING)
			num_curve->enable();
		else
			num_curve->disable();

		if ((choice->get_int_val()==ROTATION)||(choice->get_int_val()==SWINGING))
		{
			closed->disable();
			closed->set_int_val(false);
		}
		else
			closed->enable();

		if ((type==ROTATION) || (type==SWINGING))
		{
			GLfloat temp;
			for (i=0;i<curve[0].ncp;i++)
				{
					temp = curve[0].CP[i*curve[0].dim+2];
					curve[0].CP[i*curve[0].dim+2] = curve[0].CP[i*curve[0].dim+1];
					curve[0].CP[i*curve[0].dim+1] = temp;
				}
		}

		GLfloat dist_vector[3];
		GLfloat d;
		GLfloat* skcurve[MAX_SECTIONS];

		switch (type)	//in base al tipo di operazione richiama l'opportuna funzione per la
		{				//generazione della superficie
		case EXTRUSION:

			for (i=0;i<3;i++)
				dist_vector[i] = dist[i]->get_float_val();
			d = pow(dist_vector[0],2)+pow(dist_vector[1],2)+pow(dist_vector[2],2);
			if (d!=0) d=sqrt(d);
			extrusion(&surf,curve[0],dist_vector,d);
			break;
		case RULED:
			for (i=0;i<3;i++)
				dist_vector[i] = dist[i]->get_float_val();
			if (dim==3)
				for ( i=0;i<curve[1].ncp;i++)
					for (j=0;j<3;j++)
						curve[1].CP[3*i+j] += dist_vector[j];
			else
				for (i=0;i<curve[1].ncp;i++)
				{
					for (j=0;j<3;j++)
						CPtemp[4*i+j] = curve[1].CP[4*i+j] + dist_vector[j];
					CPtemp[4*i+j] = curve[1].CP[4*i+3];
				}
			ruled(&surf,curve[0].CP,CPtemp,curve[0].K,curve[0].nk,curve[0].ncp,curve[0].dim);

			break;
		case ROTATION:
			rotation(&surf,curve[0]);

			break;
		case SKINNING:
			for (i=0;i<ncurve;i++)
				skcurve[i] = curve[i].CP;
			skinning(&surf,skcurve,ncurve,curve[0].K,curve[0].nk,curve[0].ncp,distance,orderv->get_int_val(),curve[0].dim);
                
			break;

		case SWINGING:
			swinging(&surf,curve[0],curve[1],1);

			break;
		default:

			break;
		}

		if ((type==ROTATION) || (type==SWINGING))
		{
			GLfloat temp;
			for (i=0;i<curve[0].ncp;i++)
				{
					temp = curve[0].CP[i*curve[0].dim+2];
					curve[0].CP[i*curve[0].dim+2] = curve[0].CP[i*curve[0].dim+1];
					curve[0].CP[i*curve[0].dim+1] = temp;
				}
		}

		glutSetWindow(main_win);
		if (glIsList(surflist)==GL_TRUE)
			glDeleteLists(surflist,1);
		surflist = createlist(sobj,surf);

		save_obj->enable();
		save_path->enable();
		modify->enable();

		display_mode = SURF_MODE;
		
		if ( glutGetWindow() != main_win ) 
			glutSetWindow(main_win);
		update_projection(); 
		glutPostRedisplay();
		
		break;
	case ORDERU:		//controlla che l'ordine scelto sia compatibile col numero di cp
		if (orderu->get_int_val() > curve[curvei].ncp)
			orderu->set_int_val(curve[curvei].ncp);
		update_knot_vector(orderu->get_int_val(),curve[curvei].ncp);
		if (type==SWINGING)
		{
			load_knot_vector(curvei,orderu->get_int_val(),curve[curvei].ncp);
			curve[curvei].order = orderu->get_int_val();
		}
		else
			for (i=0;i<ncurve;i++)
			{
				load_knot_vector(i,orderu->get_int_val(),ncp);
				curve[i].order = orderu->get_int_val();
			}
		glutSetWindow(main_win);
		glutPostRedisplay();
		break;
	case CHOICE:		//aggiorna la vista in base al tipo di operazione scelto
		if (choice->get_int_val()==SKINNING)
			num_curve->enable();
		else
			num_curve->disable();

		if ((choice->get_int_val()==ROTATION)||(choice->get_int_val()==SWINGING))
		{
			closed->disable();
			closed->set_int_val(false);
		}
		else
			closed->enable();
		break;
	case CURVE:			//aggiorna la vista in base alla selezione della curva
		if (type==SWINGING)
		{
			orderu->set_int_val(curve[curvei].order);
			if (curvei==0)
				max_x = 0;
			else
				max_x = DIMGRID;
		}
		reload_knots(curvei,orderu->get_int_val(),curve[curvei].ncp);
		reload_weights(curvei,curve[curvei].ncp);
		if (type==SKINNING)
			sk_dist->set_float_val(distance[curvei]);
		glutSetWindow(main_win);
		glutPostRedisplay();
		break;
	case SK_DIST:		//aggiorna la distanza tra curve skinning controllando che non ci siano sovrapposizioni
		skd = sk_dist->get_float_val();
		distance[curvei] = skd;
		if (curvei==0)
		{
			if (skd > distance[1]-.1f)
				distance[0] = distance[1]-.1f;
		}
		else if (curvei==ncurve-1)
		{
			if (skd < distance[ncurve-2]+.1f)
				distance[ncurve-1] = distance[ncurve-2]+.1f;
		}
		else
		{
			if (skd < distance[curvei-1]+.1f)
				distance[curvei] = distance[curvei-1] + .1f;
			else if (skd > distance[curvei+1]-.1f)
				distance[curvei] = distance[curvei+1] - .1f;
		}
		sk_dist->set_float_val(distance[curvei]);
		break;

	case SHOWALL:		//mostra o nasconde la visione di tutte le curve sezione
		glutSetWindow(main_win);
		glutPostRedisplay();
		break;

	case QUIT:			//esce dal programma
		exit(0);
		break;

	default:
		if ((id < KNOT_ELEM+MAX_ORDER+MAX_NCP)&&(id>=KNOT_ELEM))	//variazione di un nodo
		{
			change_knot(id - KNOT_ELEM,orderu->get_int_val(),curve[curvei].ncp);
			if (type==SWINGING)
                curve[curvei].K[id - KNOT_ELEM] = knots[id - KNOT_ELEM]->get_float_val();
			else
				for (i=0;i<ncurve;i++)
					curve[i].K[id - KNOT_ELEM] = knots[id - KNOT_ELEM]->get_float_val();
			glutSetWindow(main_win);
			glutPostRedisplay();
		}
		if ((id < WEIGHT_ELEM+MAX_NCP)&&(id>=WEIGHT_ELEM))	//variazione di un peso
		{
			load_weight(cross_curve->get_int_val()-1,id - WEIGHT_ELEM);
			glutSetWindow(main_win);
			glutPostRedisplay();
		}

		break;
	}
}

//*** GLUT Callbacks

void myReshape(int w,int h)
{
	update_projection();
}

void myDisplay(void)
{

	switch(display_mode)
	{
	case SURF_MODE:		//visualizzazione della superficie generata
		
		glClearColor(0,0,0,0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();

		glEnable(GL_LIGHTING);
		glEnable(GL_DEPTH_TEST);

		glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,mat0_amb);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,mat0_diff);
		glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat0_spec);
		glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,mat0_shine);

		glTranslatef(tranXY[0],tranXY[1],tranZ[0]);
		glTranslatef(0,0,-5);
		glMultMatrixf(rot);

		if (glIsList(surflist)==GL_TRUE)
			glCallList(surflist);

		break;

	case CURVE_MODE:	//visualizzazione delle curve sezione

		glClearColor(0,0,1,0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();

		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);

		draw_grid();

		if (showall->get_int_val()==false)
			draw_curve(cross_curve->get_int_val()-1);
		else
		{
			int ci = cross_curve->get_int_val()-1;
			for (int i=0;i<ncurve;i++)
				if (i!=ci) draw_curve(i);
			draw_curve(ci);
		}

		if ((type==ROTATION)||((type==SWINGING)&&(cross_curve->get_int_val()-1==0)))
		{
			glColor3f(0.0f,0.0f,.5f);
			glBegin(GL_QUADS);
				glVertex3f(0.0f,DIMGRID,0.0f);
				glVertex3f(DIMGRID,DIMGRID,0.0f);
				glVertex3f(DIMGRID,-DIMGRID,0.0f);
				glVertex3f(0.0f,-DIMGRID,0.0f);
			glEnd();
		}

		break;

	default:
		exit(1);
	}

	glutSwapBuffers();
}

void main(int argc, char** argv)
{
	//***GLUT Init***
	int width=800,height=800;
	glutInit(&argc,argv);

	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(width,height);
	main_win = glutCreateWindow("Nurbs Tool");

	//***Setup delle luci
	glLightfv(GL_LIGHT0,GL_AMBIENT,light0_amb);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,light0_diff);
	glLightfv(GL_LIGHT0,GL_SPECULAR,light0_spec);
	glLightfv(GL_LIGHT0,GL_POSITION,light0_pos);
	glEnable(GL_LIGHT0);

	glLightfv(GL_LIGHT1,GL_AMBIENT,light1_amb);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,light1_diff);
	glLightfv(GL_LIGHT1,GL_SPECULAR,light1_spec);
	glLightfv(GL_LIGHT1,GL_POSITION,light1_pos);
	glEnable(GL_LIGHT1);

	glEnable(GL_LIGHTING);
	
	glShadeModel(GL_SMOOTH);
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);

	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	surf.CP = NULL;
	surf.U = NULL;
	surf.V = NULL;

	for (int i=0;i<MAX_SECTIONS;i++)
	{
		curve[i].CP = NULL;
		curve[i].K = NULL;
	}

	sobj = gluNewNurbsRenderer();

	gluNurbsProperty(sobj,GLU_DISPLAY_MODE,GLU_FILL);
	gluNurbsProperty(sobj,GLU_SAMPLING_TOLERANCE,20);

	//***GLUT Callbacks***
	glutDisplayFunc(myDisplay);
	glutMotionFunc(motionFunc);

	//GLUI_Master.set_glutKeyboardFunc(myGlutKeyboard);
	//GLUI_Master.set_glutSpecialFunc(myGlutSpecial);
	GLUI_Master.set_glutMouseFunc(mouseFunc);
	GLUI_Master.set_glutReshapeFunc(myReshape);
	GLUI_Master.set_glutIdleFunc(NULL);

	//***Creazione pannello destro GLUI
	right_panel = GLUI_Master.create_glui_subwindow(main_win,GLUI_SUBWINDOW_RIGHT);
	choice = right_panel->add_listbox("Cross Section",NULL,CHOICE,GLUI_handler);
	choice->add_item(EXTRUSION,"Extrusion");
	choice->add_item(RULED,"Ruled");
	choice->add_item(ROTATION,"Rotation");
	choice->add_item(SKINNING,"Skinning");
	choice->add_item(SWINGING,"Swinging");
	num_curve = right_panel->add_spinner("Curves",GLUI_SPINNER_INT);
	num_curve->set_int_limits(2,MAX_SECTIONS);
	num_curve->set_speed(.1f);
	num_curve->disable();
	rational = right_panel->add_checkbox("Razionale");
	rational->set_int_val(true);
	closed = right_panel->add_checkbox("Curva chiusa");
	closed->set_int_val(false);
	GLUI_Button* start = right_panel->add_button("Crea",START,GLUI_handler);
	modify = right_panel->add_button("Modifica",MODIFY,GLUI_handler);
	save_path = right_panel->add_edittext("Percorso",GLUI_EDITTEXT_TEXT,NULL,SAVE_PATH,GLUI_handler);
	save_path->set_text("curve1");
	save_path->disable();
	save_obj = right_panel->add_button("Salva",SAVE,GLUI_handler);
	save_obj->disable();
	modify->disable();
	GLUI_Button* quit = right_panel->add_button("Esci",QUIT,GLUI_handler);

	//***Creazione pannello inferiore GLUI
	foot_panel = GLUI_Master.create_glui_subwindow(main_win,GLUI_SUBWINDOW_BOTTOM);
	GLUI_Translation* translationXY = foot_panel->add_translation("XY Translation",GLUI_TRANSLATION_XY,tranXY);
	translationXY->set_speed(.1);
	foot_panel->add_column(false);
	GLUI_Translation* translationZ = foot_panel->add_translation("Z Translation",GLUI_TRANSLATION_Z,tranZ);
	translationZ->set_speed(.1);
	foot_panel->add_column(false);
	GLUI_Rotation* rotation = foot_panel->add_rotation("Rotation",rot);
	rotation->reset();

	//***Creazione finestra opzioni curve sezione
	create_curve_window();
	option_win->hide();

	//***Link del pannello alle finestre
	right_panel->set_main_gfx_window(main_win);
	foot_panel->set_main_gfx_window(main_win);

	glutMainLoop();
}