#include "snurbs.h"

int writeSnurbs(char* nomefile, struct snurbs surf)
{
	int i,j;
	FILE* snurbsfile;
	char nome[30];
	char* pos = strrchr(nomefile,'/');

    if (pos != NULL)
		strcpy(nome,pos+1);
	else
		strcpy(nome,nomefile);

	if ((snurbsfile = fopen(nomefile,"w")) == NULL)
		return 0;
	if ((fprintf(snurbsfile,"FILENAME:%s\n",nome) < 0)||
		(fprintf(snurbsfile,"DEGREE_U_V\n    %d    %d\n",surf.orderu-1,surf.orderv-1) < 0)||
		(fprintf(snurbsfile,"N.C.P._U_V\n    %d    %d\n",surf.ncpu,surf.ncpv) < 0)||
		(fprintf(snurbsfile,"N.KNOTS_U_V\n    %d    %d\n",surf.nu,surf.nv) < 0)||
		(fprintf(snurbsfile,"COORD.C.P.(X,Y,Z,W)\n") < 0))
	{
		fclose(snurbsfile);
		return 0;
	}
	switch (surf.dim)
	{
	case 4:
		for (i=0;i<surf.ncpu;i++)
			for (j=0;j<surf.ncpv;j++)
				if (fprintf(snurbsfile,"%#e %#e %#e %#e\n",surf.CP[4*(surf.ncpu*j+i)],
					surf.CP[4*(surf.ncpu*j+i)+1],surf.CP[4*(surf.ncpu*j+i)+2],
					surf.CP[4*(surf.ncpu*j+i)+3]) < 0)
				{
					fclose(snurbsfile);
					return 0;
				}
		break;
	case 3:
		for (i=0;i<surf.ncpu;i++)
			for (j=0;j<surf.ncpv;j++)
				if (fprintf(snurbsfile,"%#e %#e %#e %#e\n",surf.CP[3*(surf.ncpu*j+i)],
					surf.CP[3*(surf.ncpu*j+i)+1],surf.CP[3*(surf.ncpu*j+i)+2],1.0) < 0)
				{
					fclose(snurbsfile);
					return 0;
				}
		break;
	case 2:
		for (i=0;i<surf.ncpu;i++)
			for (j=0;j<surf.ncpv;j++)
				if (fprintf(snurbsfile,"%#e %#e %#e %#e\n",surf.CP[2*(surf.ncpu*j+i)],
					surf.CP[2*(surf.ncpu*j+i)+1],0.0,1.0) < 0)
				{
					fclose(snurbsfile);
					return 0;
				}
		break;
	default:
		fclose(snurbsfile);
		return 0;
	}
	if (fprintf(snurbsfile,"KNOTS_U\n") < 0)
	{
		fclose(snurbsfile);
		return 0;
	}
	for (i=0;i<surf.nu;i++)
		if (fprintf(snurbsfile,"%#e\n",surf.U[i]) < 0)
		{
			fclose(snurbsfile);
			return 0;
		}
	if (fprintf(snurbsfile,"KNOTS_V\n") < 0)
	{
		fclose(snurbsfile);
		return 0;
	}
	for (i=0;i<surf.nv;i++)
		if (fprintf(snurbsfile,"%#e\n",surf.V[i]) < 0)
		{
			fclose(snurbsfile);
			return 0;
		}
	
	fclose(snurbsfile);
	return 1;
}

int readSnurbs(char* nomefile, struct snurbs* surf)
{
	FILE* snurbsfile;
	char buf[100];
	int i,j;
	char nome[50] = "FILENAME:";
	char* pos = strrchr(nomefile,'/');
	if (pos!=NULL)
		strcat(nome,pos+1);
	else
		strcat(nome,nomefile);
	strcat(nome,"\n");

	if ((snurbsfile = fopen(nomefile,"r")) == NULL)
		return 0;
	if ((fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,nome)!=0))
	{
		fclose(snurbsfile);
		return 0;
	}
	if ((fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"DEGREE_U_V\n")!=0))
	{
		fclose(snurbsfile);
		return 0;
	}
	if (fscanf(snurbsfile," %d %d\n",&(surf->orderu),&(surf->orderv))!=2)
	{
		fclose(snurbsfile);
		return 0;
	}
	surf->orderu++;
	surf->orderv++;
	if ((fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"N.C.P._U_V\n")!=0)||
	(fscanf(snurbsfile," %d %d\n",&(surf->ncpu),&(surf->ncpv))!=2)||
	(fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"N.KNOTS_U_V\n")!=0)||
	(fscanf(snurbsfile," %d %d\n",&(surf->nu),&(surf->nv))!=2)||
	(fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"COORD.C.P.(X,Y,Z,W)\n")!=0))
	{
		fclose(snurbsfile);
		return 0;
	}

	if (((surf->U = malloc(surf->nu*sizeof(GLfloat))) == NULL)||
	((surf->V = malloc(surf->nv*sizeof(GLfloat))) == NULL)||
	((surf->CP = malloc(surf->ncpu*surf->ncpv*4*sizeof(GLfloat))) == NULL))
	{
		free(surf->U);free(surf->V);free(surf->CP);
		fclose(snurbsfile);
		return 0;
	}

	for (i=0;i<surf->ncpu;i++)
		for (j=0;j<surf->ncpv;j++)
			if ((fscanf(snurbsfile," %e %e %e %e\n",
				&(surf->CP[4*(surf->ncpu*j+i)]),&(surf->CP[4*(surf->ncpu*j+i)+1]),
				&(surf->CP[4*(surf->ncpu*j+i)+2]),&(surf->CP[4*(surf->ncpu*j+i)+3]))!=4))
			{
				free(surf->U);free(surf->V);free(surf->CP);
				fclose(snurbsfile);
				return 0;
			}
	if ((fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"KNOTS_U\n")!=0))
	{
		free(surf->U);free(surf->V);free(surf->CP);
		fclose(snurbsfile);
		return 0;
	}
	for (i=0;i<surf->nu;i++)
		if (fscanf(snurbsfile," %e\n",&(surf->U[i]))!=1)
		{
			free(surf->U);free(surf->V);free(surf->CP);
			fclose(snurbsfile);
			return 0;
		}
	if ((fgets(buf,100,snurbsfile) == NULL)||(strcmp(buf,"KNOTS_V\n")!=0))
	{
		free(surf->U);free(surf->V);free(surf->CP);
		fclose(snurbsfile);
		return 0;
	}
	for (i=0;i<surf->nv;i++)
		if (fscanf(snurbsfile," %e\n",&(surf->V[i]))!=1)
		{
			free(surf->U);free(surf->V);free(surf->CP);
			fclose(snurbsfile);
			return 0;
		}
	surf->dim=4;
	fclose(snurbsfile);
	return 1;
}

GLuint createlist(GLUnurbsObj* sobj, struct snurbs surf)
{
	GLuint list=glGenLists(1); // display list che disegna la spline
	GLuint map=0;
	int i,j,k;
	GLfloat* cpoints = malloc(surf.dim*surf.ncpu*surf.ncpv*sizeof(GLfloat));

	switch (surf.dim)
	{
	case 4:
		map = GL_MAP2_VERTEX_4;
		break;
	case 3:
		map = GL_MAP2_VERTEX_3;
		break;
	default:
		return 0;
	}

	if (surf.dim==3)
		for (i=0;i<surf.ncpv;i++)
			for (j=0;j<surf.ncpu;j++)
				for (k=0;k<3;k++)
					cpoints[3*(surf.ncpu*i+j)+k] = surf.CP[3*(surf.ncpu*i+j)+k];
	else
		for (i=0;i<surf.ncpv;i++)
			for (j=0;j<surf.ncpu;j++)
			{
				for (k=0;k<3;k++)
					cpoints[4*(surf.ncpu*i+j)+k] = surf.CP[4*(surf.ncpu*i+j)+k]*surf.CP[4*(surf.ncpu*i+j)+3];
				cpoints[4*(surf.ncpu*i+j)+3] = surf.CP[4*(surf.ncpu*i+j)+3];
			}

	glNewList(list,GL_COMPILE);
	gluBeginSurface(sobj);
		gluNurbsSurface(sobj,surf.nu,surf.U,surf.nv,surf.V,surf.dim,surf.dim*surf.ncpu,cpoints,
			surf.orderu,surf.orderv,map);
	gluEndSurface(sobj);
	glEndList();

	free(cpoints);
	return list;
}

GLuint createlistcurve(GLUnurbsObj* cobj, struct cnurbs curve)
{
	GLuint list=glGenLists(1);
	GLuint map=0;
	int i,j;
	GLfloat* cpoints = malloc(curve.dim*curve.ncp*sizeof(GLfloat)); 

	switch (curve.dim)
	{
	case 4:
		map = GL_MAP1_VERTEX_4;
		break;
	case 3:
		map = GL_MAP1_VERTEX_3;
		break;
	default:
		return 0;
	}

	if (curve.dim==3)
		for (i=0;i<curve.ncp;i++)
			for (j=0;j<3;j++)
				cpoints[3*i+j] = curve.CP[3*i+j];
	else
		for (i=0;i<curve.ncp;i++)
		{
			for (j=0;j<3;j++)
				cpoints[4*i+j] = curve.CP[4*i+j]*curve.CP[4*i+3];
			cpoints[4*i+3] = curve.CP[4*i+3];
		}
    
	glNewList(list,GL_COMPILE);
	gluBeginCurve(cobj);
		gluNurbsCurve(cobj,curve.nk,curve.K,curve.dim,cpoints,curve.order,map);
	gluEndCurve(cobj);
	glEndList();

	free(cpoints);

	return list;
}