#pragma once

#ifndef _SNURBS
#define _SNURBS

#include "glut.h"
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#ifdef __cplusplus
extern "C" {
#endif

//********************************************************************
// struttura snurbs che contiene i parametri di una superficie NURBS
//********************************************************************
struct snurbs
{
	int orderu;			// ordine nella coordinata u
	int orderv;			// ordine nella coordinata v
	int ncpu;			// numero di control points nella coordinata u
	int ncpv;			// numero di control points nella coordinata v
	int nu;				// numero di elementi del vettore nodale in u
	int nv;				// numero di elementi del vettore nodale in v
	int dim;			// 3: superficie non razionale 4: superficie razionale
	GLfloat* U;			// puntatore al vettore nodale U[nu]
	GLfloat* V;			// puntatore al vettore nodale V[nv]
	GLfloat* CP;		// puntatore al vettore dei control points CP[ncpu][ncpv][dim]
};

//********************************************************************
// struttura cnurbs che contiene i parametri di una curva NURBS
//********************************************************************

struct cnurbs
{
	int order;			// ordine della curva
	int ncp;			// numero di control points
	int nk;				// numero di elementi del vettore nodale
	int dim;			// 3: superficie non razionale 4: superficie razionale
	GLfloat* K;			// puntatore al vettore nodale K[nk]
	GLfloat* CP;		// puntatore al vettore dei control points CP[ncp][dim]
};

//********************************************************************
// macro che liberano la memoria dei vettori nelle strutture
//********************************************************************

#define CNURBS_FREE(curve) free(curve.K);free(curve.CP)
#define SNURBS_FREE(surf) free(surf.U);free(surf.V);free(surf.CP)

int writeSnurbs(char* nomefile, struct snurbs surf);
// salva la superficie contenuta nella struttura snurbs in un file in formato snurbs
// Input:
// nomefile stringa contenente il nome del file snurbs
// surf struttura contenente la superficie snurbs
// Output:
// restituisce 0 se c' stato un errore e 1 altrimenti

int readSnurbs(char* nomefile, struct snurbs* surf);
// carica la superficie contenuta in un file snurbs nella struttura snurbs
// Input:
// nomefile stringa contenente il nome del file snurns
// Output:
// surf stringa contenente la superficie snurbs
// restiuisce 0 se c stato un errore e 1 altrimenti

GLuint createlist(GLUnurbsObj* sobj, struct snurbs surf);
// crea una display list a partire da una struttura snurbs e un oggetto superficie
//Input:
// sobj oggetto glu contenente le propriet di rendering della superficie
// surf struttura contenente la superficie
//Output:
// restituisce la display list generata o 0 se c' stato un errore

GLuint createlistcurve(GLUnurbsObj* cobj, struct cnurbs curve);
// crea una display list a partire da una struttura cnurbs e un oggetto curva
//Input:
// cobj oggetto glu contenente le propriet di rendering della curva
// curve struttura contenente la curva
//Output:
// restituisce la display list generata o 0 se c' stato un errore

#ifdef __cplusplus
}
#endif

#endif