#include "spline.h"

int find_node(GLfloat* U, int nu, GLfloat u, int deg)
{
	int order = deg+1;
	int left=deg, right=nu-deg-2, mid=(int)floor((left+right)/2);
	if (u==0)
		return left;
	if (u==1)
		return right;
	// ricerca per bisezione
	while ((u<U[mid])||(u>=U[mid+1]))
	{
		if (u<U[mid])
			right=mid;
		else
			left=mid+1;
		mid=(int)floor((left+right)/2);
	}
	return mid;
}

void spline_eval_db(struct cnurbs curve, GLfloat u, GLfloat* s)
{
	int i,j,k;
	GLfloat* c=malloc(curve.order*sizeof(GLfloat));
	int node;
	if (u<0) u=0;
	if (u>1) u=1;
	node=find_node(curve.K,curve.nk,u,curve.order-1); // calcolo della posizione nel vettore nodale

	for (i=0;i<curve.dim;i++)
	{
		if ((curve.dim == 3)||((curve.dim==4)&&(i==3)))
			for (j=0;j<curve.order;j++)
				c[j]=curve.CP[curve.dim*(node-curve.order+1+j)+i]; // vettore dei punti di controllo da node-order+1 a node
		else if (curve.dim==4)
			for (j=0;j<curve.order;j++)
				c[j]=curve.CP[curve.dim*(node-curve.order+1+j)+3]*
				curve.CP[curve.dim*(node-curve.order+1+j)+i];
		else
			return;

		for (j=1;j<curve.order;j++)
		{
			for (k=curve.order-1;k>=j;k--) // formula di de Boor
				c[k]=((u-curve.K[k+node-curve.order+1])*c[k]+(curve.K[k+node-j+1]-u)*c[k-1])/
				(curve.K[k+node-j+1]-curve.K[k+node-curve.order+1]);
		}
		s[i]=c[curve.order-1]; //  il punto della curva cercato
	}
	free(c);
}

int find_spline_base(GLfloat* U, int nu, GLfloat u, int order, GLfloat* base)
{
	int node=find_node(U,nu,u,order-1); // calcolo della posizione nel vettore nodale
	int i,j;
	GLfloat sum,left,right;
	base[order-1]=1; // N(i,0)=1 <=> i=node
	for (i=0;i<order-1;i++)
		base[i]=0;
	for (i=1;i<order;i++) // formula ricorsiva per il calcolo della base spline
	{
		sum=0;
		for (j=order-1;j>=(order-1-i);j--)
		{
			left=(u-U[j+node-order+1])*base[j];
			right=(U[j+node+2-order+i]-u)*sum;
			if (left!=0)
				left/=U[j+node-order+i+1]-U[j+node-order+1];
			if (right!=0)
				right/=(U[j+node+2-order+i]-U[j+node-order+2]);
			sum=base[j];
			base[j]=left+right;
		}
	}
	return node;
}

int find_spline_base_dev(GLfloat* U, int nu, GLfloat u, int order, GLfloat* base, GLfloat* devbase)
{
	int node=find_node(U,nu,u,order-1); // calcolo della posizione nel vettore nodale
	int i,j;
	GLfloat sum,left,right;
	base[order-1]=1; // N(i,0)=1 <=> i=node
	for (i=0;i<order-1;i++)
		base[i]=0;
	for (i=1;i<order-1;i++) // formula ricorsiva per il calcolo della base spline
	{
		sum=0;
		for (j=order-1;j>=(order-1-i);j--)
		{
			left=(u-U[j+node-order+1])*base[j];
			right=(U[j+node+2-order+i]-u)*sum;
			if (left!=0)
				left/=U[j+node-order+i+1]-U[j+node-order+1];
			if (right!=0)
				right/=(U[j+node+2-order+i]-U[j+node-order+2]);
			sum=base[j];
			base[j]=left+right;
		}
	}
	
	sum = 0;
	for (j=order-1;j>=0;j--)
	{
		left=base[j];
		right=sum;
		if (left!=0)
			left/=U[j+node]-U[j+node-order+1];
		if (right!=0)
			right/=(U[j+node+1]-U[j+node-order+2]);
		sum=base[j];
		devbase[j]=(order-1)*(left-right);
	}

	sum = 0;
	for (j=order-1;j>=0;j--)
	{
		left=(u-U[j+node-order+1])*base[j];
		right=(U[j+node+2-1]-u)*sum;
		if (left!=0)
			left/=U[j+node]-U[j+node-order+1];
		if (right!=0)
			right/=(U[j+node+1]-U[j+node-order+2]);
		sum=base[j];
		base[j]=left+right;
	}
	return node;
}

void spline_eval_1D(struct cnurbs curve, GLfloat u, GLfloat* s)
{
	int i,j;
	GLfloat* base=malloc(curve.order*sizeof(GLfloat));
	int node;
	if (u<0) u=0;
	if (u>1) u=1;
	node = find_spline_base(curve.K,curve.nk,u,curve.order,base); // calcolo della base spline

	for (i=0;i<curve.dim;i++)
	{
		s[i]=0;
		if ((curve.dim==3)||((curve.dim==4)&&(i==3)))
			for (j=0;j<curve.order;j++)
				s[i]+=base[j]*curve.CP[curve.dim*(j+node-curve.order+1)+i]; // valutazione s(u)=sum(c*N(i,order),i=node-order+1:node)
		else if (curve.dim==4)
			for (j=0;j<curve.order;j++)
				s[i]+=base[j]*curve.CP[curve.dim*(j+node-curve.order+1)+3]*
				curve.CP[curve.dim*(j+node-curve.order+1)+i]; // valutazione s(u)=sum(c*N(i,order),i=node-order+1:node)
		else
			return;
	}
	free(base);
}

void spline_eval_1D_t(struct cnurbs curve, GLfloat u, GLfloat* p, GLfloat* t)
{
	int i,j;
	GLfloat* base=malloc(curve.order*sizeof(GLfloat));
	GLfloat* devbase=malloc(curve.order*sizeof(GLfloat));
	GLfloat D[3];
	GLfloat omega;
	GLfloat d_D[3];
	GLfloat d_omega;
	int node;
	if (u<0) u=0;
	if (u>1) u=1;
	node = find_spline_base_dev(curve.K,curve.nk,u,curve.order,base,devbase); // calcolo della base spline

	if (curve.dim==4)
	{
		for (i=0;i<3;i++)
		{
			D[i]=0;
			for (j=0;j<curve.order;j++)
				D[i]+=base[j]*curve.CP[curve.dim*(j+node-curve.order+1)+3]*
				curve.CP[curve.dim*(j+node-curve.order+1)+i]; // valutazione s(u)=sum(c*N(i,order),i=node-order+1:node)
		}

		omega=0;
		for (i=0;i<curve.order;i++)
			omega+=base[i]*curve.CP[curve.dim*(i+node-curve.order+1)+3];

		for (i=0;i<3;i++)
		{
			d_D[i]=0;
			for (j=0;j<curve.order;j++)
				d_D[i]+=devbase[j]*curve.CP[curve.dim*(j+node-curve.order+1)+3]*
				curve.CP[curve.dim*(j+node-curve.order+1)+i];
		}

		d_omega=0;
		for (i=0;i<curve.order;i++)
			d_omega+=devbase[i]*curve.CP[curve.dim*(i+node-curve.order+1)+3];

		for (i=0;i<3;i++)
			p[i] = D[i]/omega;

		for (i=0;i<3;i++)
			t[i] = (d_D[i]*omega - D[i]*d_omega)/pow(omega,2);
	}
	else if (curve.dim==3)
	{

		for (i=0;i<3;i++)
		{
			D[i]=0;
			for (j=0;j<curve.order;j++)
				D[i]+=base[j]*curve.CP[curve.dim*(j+node-curve.order+1)+i]; // valutazione s(u)=sum(c*N(i,order),i=node-order+1:node)
		}

		for (i=0;i<3;i++)
		{
			d_D[i]=0;
			for (j=0;j<curve.order;j++)
				d_D[i]+=devbase[j]*curve.CP[curve.dim*(j+node-curve.order+1)+i];
		} 

		for (i=0;i<3;i++)
			p[i] = D[i];

		for (i=0;i<3;i++)
			t[i] = d_D[i];
	}
	else
		return;

	free(base);
	free(devbase);
}

void spline_eval_2D(struct snurbs surf, GLfloat u, GLfloat v, GLfloat* s)
{
	int i,j,k;
	GLfloat* baseu=malloc(surf.orderu*sizeof(GLfloat));
	GLfloat* basev=malloc(surf.orderv*sizeof(GLfloat));
	int nodeu;
	int nodev; 

	if (u<0) u=0;
	if (u>1) u=1;
	if (v<0) v=0;
	if (v>1) v=1;

	nodeu = find_spline_base(surf.U,surf.nu,u,surf.orderu,baseu); // calcolo della base spline in u
	nodev = find_spline_base(surf.V,surf.nv,v,surf.orderv,basev); // calcolo della base spline in v
	for (i=0;i<surf.dim;i++)
	{
		if ((surf.dim==3)||((surf.dim==4)&&(i==3)))
		{
			s[i]=0;
			for (j=0;j<surf.orderu;j++)
			{
				for (k=0;k<surf.orderv;k++) // valutazione della spline
					s[i]+=baseu[j]*basev[k]*surf.CP[surf.dim*((j+nodeu-surf.orderu+1)+
					surf.ncpu*(k+nodev-surf.orderv+1))+i];
			}
		}
		else if (surf.dim==4)
		{
			s[i]=0;
			for (j=0;j<surf.orderu;j++)
			{
				for (k=0;k<surf.orderv;k++) // valutazione della spline
					s[i]+=baseu[j]*basev[k]*surf.CP[surf.dim*((j+nodeu-surf.orderu+1)+
					surf.ncpu*(k+nodev-surf.orderv+1))+3]*surf.CP[surf.dim*((j+nodeu-surf.orderu+1)+
					surf.ncpu*(k+nodev-surf.orderv+1))+i];
			}
		}
		else
			return;
	}
	free(baseu);
	free(basev);
}