#pragma once

#ifndef _SPLINE_H
#define _SPLINE_H

#include <math.h>
#include <malloc.h>
#include "glut.h"

#include "snurbs.h"

#ifdef __cplusplus
extern "C" {
#endif

int find_node(GLfloat* U, int nu, GLfloat u, int deg);
// Calcola il nodo a cui appartiene il parametro u asseganto.
// Input:
// U[nu] vettore nodale
// nu dimensione vettore nodale
// u parametro che identifica il punto della curva
// deg grado della curva
// Output:
// posizione nel vettore nodale
void spline_eval_db(struct cnurbs curve, GLfloat u, GLfloat* s);
// Valuta la curva spline usando l'algoritmo di de Boor
// Input:
// curve curva spline da valutare
// u parametro che identifica il punto della curva
// Output:
// s[dim] vettore contenente il punto della curva valutato
int find_spline_base(GLfloat* U, int nu, GLfloat u, int order, GLfloat* base);
// Trova gli elementi della base spline non nulli quando il parametro  u
// Input:
// U[nu] vettore nodale
// nu numero dei nodi
// u parametro che identifica il punto della curva
// order ordine della curva
// Output:
// posizione nel vettore nodale (i)
// base[order] elementi non nulli N(i-order+1),...,N(i)
void spline_eval_1D(struct cnurbs curve, GLfloat u, GLfloat* s);
// Valuta una curva con parametro u calcolando le basi spline non nulle
// Input:
// curve curva spline da valutare
// u parametro che identifica il punto della curva
// Output:
// s[dim] punto della curva valutato
void spline_eval_2D(struct snurbs surf, GLfloat u, GLfloat v, GLfloat* s);
// Valuta una superficie con parametri u,v calcolando le basi spline non nulle
// Input:
// surf superficie spline da valutare
// u,v parametri che identificano il punto della superficie
// Output:
// s[dim] punto della superficie valutato
void spline_eval_1D_t(struct cnurbs curve, GLfloat u, GLfloat* p, GLfloat* t);
// Valuta una curva con parametro u calcolando le basi spline non nulle
// Input:
// curve curva spline da valutare
// u parametro che identifica il punto della curva
// Output:
// s[3] punto della curva valutato
// t[3] tangente al punto della curva

#ifdef __cplusplus
}
#endif

#endif